//Preferred habitat and the term structure of interest rates in DSGE models
//Celso Costa


var Y C I G L K
W R RB RBL P PI CM Xi LAMBDA
T B BL U Q spread
SP SL SI A SG ST Sm Smarkup PIW
DL DT DG DY DC DI PIobs RBobs2 RBLobs2 DU;

varexo e_P e_L e_I e_A e_G e_T e_m e_BL emarkup emarkupW eDU;

parameters beta delta psi1 varphi theta alpha eta omega phiBss phiBLss phiTss psi2 chi
varthetaL phic
rhoP rhoL rhoI rhoA rhoG rhoT rhom rhoBL rhomarkup thetaW
gammaR gammaPI gammaY gammaG gammaT phiG phiT 
RBss RBLss Yss Pss phiCss phiIss phiGss phiKss;

//Parmeteres calibration****************************************************
Yss = 15.525;                                                              //15525;
beta = 1/1.000698514;                                                      //The inversion of the average 3-month maturity bond
delta = 0.025;                                                             //De Negro et al "The FRBNY DSGE Model"
psi1 = (1/beta)-(1-delta);
varphi = 7.7;                                                              //De Negro et al "The FRBNY DSGE Model"
phic = 0.7;                                                                //De Negro et al "The FRBNY DSGE Model"
phiTss = 0.35;
RBss = 1.000698514;                                                        //Average maturity title 3 months
RBLss = 1.024701063;                                                       //Average maturity title 10 years
Pss = 1;
phiCss = 0.66;
phiIss = 0.17;
phiGss = 0.18;
phiKss = 4*2.5;
varthetaL = (1/(Yss*(1.5 - beta)))*((RBLss*beta/RBss^39) - 1);
alpha = 0.33;                                                              //De Negro et al "The FRBNY DSGE Model"
eta = 2.5;                                                                 //De Negro et al "The FRBNY DSGE Model"
omega = 2;


//Estimated parameters*****************************************************
theta = 0.75;                                                              //De Negro et al "The FRBNY DSGE Model"
thetaW = 0.75;
psi2 = 4;
chi = 4;
gammaR = 0.5;                                                              //De Negro et al "The FRBNY DSGE Model"
gammaY = 0.2;                                                              //De Negro et al "The FRBNY DSGE Model"
gammaPI = 2;                                                               //De Negro et al "The FRBNY DSGE Model"

gammaG = 0;
gammaT = 0.1;
phiG = 0;
phiT = 0.1;

phiBss = 0.6; 
phiBLss = 3.4;

rhoP = 0.5;
rhoL = 0.5; 
rhoI = 0.5;
rhoA = 0.5;
rhoG = 0.5; 
rhoT = 0.5; 
rhom = 0.5;
rhoBL = 0.5;
rhomarkup = 0.5;

model(linear);
//steady state
#Xiss = beta;
#Qss = Pss;
#Css = phiCss*Yss;
#Iss = phiIss*Yss;
#Gss = phiGss*Yss;
#Kss = phiKss*Yss;
#Bss = phiBss*Yss;
#BLss = phiBLss*Yss;
#Tss = Gss - (Bss/Pss)*((1/RBss)-1) - (BLss/Pss)*((1/RBLss)-(1/RBss^39));
#Rss = Pss*((1/beta)-(1-delta));
#CMss = Pss*((varphi-1)/varphi)*(1-beta*theta);
#Wss = (1-alpha)*(CMss^(1/(1-alpha)))*((alpha/Rss)^(alpha/(1-alpha)));
#Lss = CMss*(1-alpha)*Yss/Wss;
#LAMBDAss = (L^omega)/Wss;
//1-preference shock
SP = rhoP*SP(-1) + ((1-rhoP^2)^0.5)*e_P;
//2-job offer shock
SL = rhoL*SL(-1) + ((1-rhoL^2)^0.5)*e_L;
//3-demand for consumption
(1-phic*beta)*(LAMBDA + P) = SP - (eta/(1-phic))*(C - phic*C(-1)) 
- (beta*phic/(1-phic))*(SP(+1) - eta*(C(+1) - phic*C));
//4-Job offer
//SL + omega*L = W + LAMBDA;
(1+beta*thetaW)*PIW = thetaW*PIW(-1) + beta*PIW(+1) +((1-thetaW)*(1-beta*thetaW)/thetaW)
*(SP+SL+omega*L-LAMBDA - (W - P) + emarkupW); 
PIW = W - W(-1);
//5-Demand for the money market title
0 = RB+Xi;
//6-Demand for the long-term bond
(3/2)*varthetaL*Yss*(2*(BL-BL(-1))+Y) = (RBLss*Xiss/RBss^39)*
(RBL - RB(+2) - RB(+3) - RB(+4) - RB(+5) - RB(+6) - RB(+7) - RB(+8) - RB(+9) - RB(+10)
- RB(+11)- RB(+12)- RB(+13)- RB(+14)- RB(+15)- RB(+16)- RB(+17)- RB(+18)- RB(+19)- RB(+20) 
- RB(+21)- RB(+22)- RB(+23)- RB(+24)- RB(+25)- RB(+26)- RB(+27)- RB(+28)- RB(+29)- RB(+30) 
- RB(+31)- RB(+32)- RB(+33)- RB(+34)- RB(+35)- RB(+36)- RB(+37)- RB(+38)- RB(+39)- RB(+40) - RB(+41)+ Xi) 
+ Xiss*varthetaL*Yss*(RBL - RBL(+1) + Xi + 3*(BL(+1)-BL)+Y(+1));
//7-stochastic discount
Xi = LAMBDA(+1) - LAMBDA;
//8-Intermediate goods production function
Y = A + alpha*(U+K(-1))+(1-alpha)*L;
//9-productivity shock
A = rhoA*A(-1) + ((1-rhoA^2)^0.5)*e_A;
//10-Marginal rate of substitution between inputs
L - U - K(-1) = R - W;
//11-Marginal cost of firms producing intermediate goods
CM = alpha*R + (1-alpha)*W - A;
//12-Phillips Equation
(1+beta*theta)*PI = theta*PI(-1) + beta*PI(+1) + ((1-theta)*(1-beta*theta)/theta)*(CM-P + Smarkup);
//13-law of movement of capital
K = (1-delta)*K(-1) + delta*I;
//14-Productivity shock on investment
SI = rhoI*SI(-1) + ((1-rhoI^2)^0.5)*e_I;
//15-Demand for installed capacity
(Rss/Pss)*(R - P) = psi2*U;
//16-Tobin's Q
(Qss/Xiss)*(Q - Xi) = (1-delta)*Qss*Q(+1) + Rss*(R(+1)+U(+1)) - psi1*Pss*U(+1);
//17-Euler's equation for investment
Pss*P - Qss*Q + chi*Qss*(I - I(-1) - SI) = chi*Xiss*Qss*(I(+1) - I - SI(+1));
//18-Government budget constraint
(Bss/RBss)*(B-RB)-Bss*B(-1) + (BLss/RBLss)*(BL-RBL)-(BLss/RBss^39)
*(BL(-1)- RB - RB(+2) - RB(+3) - RB(+4) - RB(+5) - RB(+6) - RB(+7) - RB(+8) - RB(+9) - RB(+10)
- RB(+11)- RB(+12)- RB(+13)- RB(+14)- RB(+15)- RB(+16)- RB(+17)- RB(+18)- RB(+19)- RB(+20) 
- RB(+21)- RB(+22)- RB(+23)- RB(+24)- RB(+25)- RB(+26)- RB(+27)- RB(+28)- RB(+29)- RB(+30) 
- RB(+31)- RB(+32)- RB(+33)- RB(+34)- RB(+35)- RB(+36)- RB(+37)- RB(+38)- RB(+39)- RB(+40)) 
= Pss*Gss*(P+G) - Pss*Tss*(P+T);
//19-Law of motion of government current expenditures
P+G = gammaG*(P(-1)+G(-1)) + (1-gammaG)*phiG*(B(-1) - Y(-1) - P(-1) + BL(-1) - Y(-1) - P(-1)) + SG;
//20-Government revenue movement law (lump-sum)
P+T = gammaT*(P(-1)+T(-1)) + (1-gammaT)*phiT*(B(-1) - Y(-1) - P(-1) + BL(-1) - Y(-1) - P(-1)) + ST;
//21-Shock in current government spending
SG = rhoG*SG(-1) + ((1-rhoG^2)^0.5)*e_G;
//22-shock to government revenue
ST = rhoT*ST(-1) + ((1-rhoT^2)^0.5)*e_T;
//23-Taylor rule
RB = gammaR*RB(-1) + (1-gammaR)*(gammaY*Y + gammaPI*PI) + Sm;
//24-monetary shock
Sm = rhom*Sm(-1) + ((1-rhom^2)^0.5)*e_m;
//25-Taxa bruta gross inflation rate
PI(+1) = P(+1) - P;
//26-Equilibrium condition in the goods market
Yss*Y = Css*C + Iss*I + Gss*G;
//27-Long-term bond supply shock
BL = rhoBL*BL(-1) + e_BL;
//28-definition of spread
spread = RBL - RB;
//29-Firm markup shock
Smarkup = rhomarkup*Smarkup(-1) + emarkup;
//endozenized variables**************************************************
//log-difference variables
DL = L - L(-1) + 0.003438; 
DT = T - T(-1) + 0.004827; 
DG = G - G(-1) + 0.005485; 
DY = Y - Y(-1) + 0.02772;
DC = C - C(-1) + 0.03021;
DI = I - I(-1) + 0.04513;
DU = U - U(-1) + 0.00032696 + eDU;
//RBobs = RB + 1.126;
//RBLobs = RBL + 1.186;
//variables in log
RBobs2 = RB + 0.1131;
RBLobs2 = RBL + 0.1657;
//PIobs2 = PI + 0.006651;
PIobs = PI + 1.0067;
end;

varobs DL DT DG DY DC DI PIobs RBobs2 RBLobs2 DU;

shocks;
var e_P; stderr 0.01;
var e_L; stderr 0.01;
var e_I; stderr 0.01;
var e_A; stderr 0.01;
var e_G; stderr 0.01;
var e_T; stderr 0.01;
var e_m; stderr 0.01;
var e_BL; stderr 0.01;
var emarkup; stderr 0.01;
end;

estimated_params;
//alpha, beta_pdf, 0.33, 0.02;
//eta, gamma_pdf, 2, 0.25;
//omega, gamma_pdf, 1.5, 0.25;
thetaW, beta_pdf, 0.7, 0.1;
theta, beta_pdf, 0.7, 0.1;
//Yss, gamma_pdf, 15.525, 1;
psi2, gamma_pdf, 4, 1.5;
chi, gamma_pdf, 4, 1.5;
gammaR, beta_pdf, 0.8, 0.01;
gammaY, gamma_pdf, 0.2, 0.01;
gammaPI, gamma_pdf, 3, 0.1;
phiBss, gamma_pdf, 0.6, 0.01;
phiBLss, gamma_pdf, 3.4, 0.1;
gammaG, uniform_pdf, , , 0.01, 0.99;
gammaT, uniform_pdf, , , 0.01, 0.99;
phiG, uniform_pdf, , , -0.5, 0;
phiT, uniform_pdf, , , 0, 1;
//autoregressive parameters***********************************************
rhoP, beta_pdf, 0.5, 0.25; 
rhoL, beta_pdf, 0.5, 0.25; 
rhoI, beta_pdf, 0.5, 0.25; 
rhoA, beta_pdf, 0.5, 0.25; 
rhoG, beta_pdf, 0.5, 0.25; 
rhoT, beta_pdf, 0.5, 0.25; 
rhom, beta_pdf, 0.5, 0.25; 
rhoBL, beta_pdf, 0.5, 0.25;
rhomarkup, beta_pdf, 0.5, 0.25;
//Standard errors of the shocks*************************************************
stderr e_P, inv_gamma_pdf, 1, inf;
stderr e_L, inv_gamma_pdf, 1, inf;
stderr e_I, inv_gamma_pdf, 1, inf;
stderr e_A, inv_gamma_pdf, 1, inf;
stderr e_G, inv_gamma_pdf, 1, inf;
stderr e_T, inv_gamma_pdf, 1, inf;
stderr e_m, inv_gamma_pdf, 1, inf;
stderr e_BL, inv_gamma_pdf, 1, inf;
stderr emarkup, inv_gamma_pdf, 1, inf;
stderr emarkupW, inv_gamma_pdf, 1, inf;
stderr eDU, inv_gamma_pdf, 1, inf;
end;

identification;
dynare_sensitivity;


// /*
estimation(datafile=term_struc_data1, first_obs=72,smoother, mh_jscale=0.3,  
plot_priors = 0, mh_drop=0.8, mode_check, mh_replic=100000,
 mh_nblocks=5, mode_compute=6, bayesian_irf, irf=12);

shock_decomposition Y PI RB;

results_HP_complete = oo_;
save results_HP_complete;

// */
